#! /usr/bin/env bash

# The script expects the dependant libraries(zmq) and executable
# jar at the same directory.

# Set the '/proc/pid/comm' to identify this script, default is 'bash'
THIS_SCRIPT_COMM="gpudb-sql-eng.sh" # Shortened, comm is only 16 chars
printf $THIS_SCRIPT_COMM > /proc/$$/comm

# The directory of this script
PROJECT_HOME="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"
#echo $PROJECT_HOME

THIS_HOSTNAME=$(hostname)

ETCD_NAME="etcd"

GPUDB_ROOT_DIR=$(readlink -e -n "$PROJECT_HOME/..")  # /opt/gpudb

GPUDB_ETCD_ROOT_DIR="$PROJECT_HOME"
GPUDB_ETCD_LOG_DIR="${GPUDB_ETCD_ROOT_DIR}/logs"

GPUDB_ETCD_LOG_FILENAME_PREFIX="etcd"
GPUDB_ETCD_LOG_FILE_PREFIX="$GPUDB_ETCD_LOG_DIR/$GPUDB_ETCD_LOG_FILENAME_PREFIX-$THIS_HOSTNAME"
GPUDB_ETCD_LOG_FILE_WITH_DATE="${GPUDB_ETCD_LOG_FILE_PREFIX}-$(date +'%Y-%m-%d_%H.%M.%S')"

GPUDB_ETCD_LOG_LINK_FILE="${GPUDB_ETCD_LOG_FILE_PREFIX}.log"
GPUDB_ETCD_OUT_LINK_FILE="${GPUDB_ETCD_LOG_FILE_PREFIX}.out"

# Can be overridden on cmd line for 'start'
GPUDB_ETCD_LOG_FILE="${GPUDB_ETCD_LOG_FILE_WITH_DATE}.log"
GPUDB_ETCD_OUT_FILE="${GPUDB_ETCD_LOG_FILE_WITH_DATE}.out"

# ---------------------------------------------------------------------------

function echo_status
{
    local COMPONENT=$1
    local STATE=$2
    printf "%-16s : $STATE" "$COMPONENT"
}

# ---------------------------------------------------------------------------

IS_RUNNING_IN_DOCKER=  # 1 if true, else 0 after calling get_is_running_in_docker
function get_is_running_in_docker
{
    if [ -z "$IS_RUNNING_IN_DOCKER" ]; then
        IS_RUNNING_IN_DOCKER=0
        if grep -qF ":/docker" /proc/$$/cgroup ; then
            IS_RUNNING_IN_DOCKER=1
        fi
    fi
}

# Echos the input list of pids and if running on host remove any pids of docker processes.
# Returns 1 if pid list is empty
function filter_docker_pids
{
    local PIDS="$@"
    local VALID_PIDS=$PIDS

    get_is_running_in_docker

    if [ ! -z "$PIDS" ] && [ $IS_RUNNING_IN_DOCKER -eq 0 ]; then
        VALID_PIDS=" "
        for pid in $(echo $PIDS); do
            if [ -f /proc/${pid}/cgroup ] && ! grep -qF ":/docker" /proc/${pid}/cgroup > /dev/null 2>&1 ; then
                VALID_PIDS+="$pid "
            fi
        done
        VALID_PIDS=${VALID_PIDS% *}
    fi

    if [ -z "$VALID_PIDS" ]; then
        return 1
    fi

    echo $VALID_PIDS
    return 0
}

# ---------------------------------------------------------------------------

ETCD_CMD="${GPUDB_ETCD_ROOT_DIR}/etcd --config-file ${GPUDB_ETCD_ROOT_DIR}/etcd.conf.yml"
function etcd_start
{
    local NOHUP="$1"
    local RET="0"

    local ETCD_PID="$(etcd_pid)"

    if [[ "a$ETCD_PID" != "a" ]]; then
        echo_status "$ETCD_NAME" " is already running...\n"
        return "1"
    fi

#    export LD_LIBRARY_PATH=$PROJECT_HOME/lib:$LD_LIBRARY_PATH

#    echo "****"
#    echo $2

    if [ ! -z "$2" ]; then
        GPUDB_ETCD_CONFIG_FILE="$2"
    fi
    
    if [ ! -z "$3" ]; then
        GPUDB_ETCD_LOG_FILE="$3"
        GPUDB_ETCD_OUT_FILE="${GPUDB_ETCD_LOG_FILE%.*}.out" # same name w/ .out extension.
    fi

    mkdir -p "${GPUDB_ETCD_LOG_DIR}"

    echo "$(date) : Starting Kinetica ETCD" > "${GPUDB_ETCD_LOG_FILE}"
    echo "$(date) : Starting Kinetica ETCD" > "${GPUDB_ETCD_OUT_FILE}"
    ln -sf "${GPUDB_ETCD_LOG_FILE}" "${GPUDB_ETCD_LOG_LINK_FILE}"
    ln -sf "${GPUDB_ETCD_OUT_FILE}" "${GPUDB_ETCD_OUT_LINK_FILE}"

    echo_status "$ETCD_NAME" "Starting\n"
    pushd $GPUDB_ETCD_ROOT_DIR > /dev/null

    echo "running command: ${ETCD_CMD}" >> ${GPUDB_ETCD_OUT_FILE}

    if [ "${NOHUP}" = "NOHUP" ]; then
        echo "nohup mode" >> "${GPUDB_ETCD_OUT_FILE}"
        nohup ${ETCD_CMD} >> "${GPUDB_ETCD_OUT_FILE}" 2>&1 &
    else
        ${ETCD_CMD} >> "$GPUDB_ETCD_OUT_FILE" 2>&1 &
        local PID=$!

        echo "PID: $PID" >> ${GPUDB_ETCD_OUT_FILE}

        if [[ "a$PID" == "a" ]]; then
            echo_status "$ETCD_NAME" "Failed to start\n" | tee -a ${GPUDB_ETCD_OUT_FILE}
            return 1
        fi

        wait $PID
        local RET="$?"

        # 0 == success, 130 == stopped via SIGINT
        if [ $RET -ne 0 ] && [ $RET -ne 130 ] && [ $RET -ne 143 ]; then
            echo_status "$ETCD_NAME" "Exited with return code $RET\n" | tee -a ${GPUDB_ETCD_OUT_FILE}
        else
            RET=0
        fi
    fi
    popd > /dev/null

    return "$RET"
}

function etcd_pid
{
    local A=$(ps auxww | grep "${ETCD_CMD}" | grep -v grep | awk '{ print $2 }')
    echo $(filter_docker_pids $A)
}

function etcd_stop
{
    local ETCD_PID="$(etcd_pid)"

    if [[ "a$ETCD_PID" == "a" ]]; then
        echo_status "$ETCD_NAME" "Stopped\n"
        return
    fi

    echo_status "$ETCD_NAME" "Stopping ($ETCD_PID) "
    kill -INT $ETCD_PID

    # Wait for EXIT_SLEEP_TIME_SECS*EXIT_SLEEP_COUNT secs showing that we're still waiting...
    local EXIT_SLEEP_TIME_SECS=1
    local EXIT_SLEEP_COUNT=120

    local counter=0
    while [ $counter -lt $EXIT_SLEEP_COUNT ]; do
        if [[ "a$(etcd_pid)" == "a" ]]; then
            echo ""
            break
        fi

        if [ $(($counter % 5)) -eq 0 ]; then
            echo -n "."
        fi
        sleep $EXIT_SLEEP_TIME_SECS
        counter=$((counter+1))
    done

    ETCD_PID=$(etcd_pid)
    if [[ "a$ETCD_PID" == "a" ]]; then
        echo_status "$ETCD_NAME" "Stopped\n"
    else
        echo_status "$ETCD_NAME" "Killed ($ETCD_PID)\n"
        kill -9 $ETCD_PID
    fi
}

function etcd_status
{
    local ETCD_PID=$(etcd_pid)

    if [[ "a$ETCD_PID" != "a" ]]; then
        echo_status "$ETCD_NAME" "Running ($ETCD_PID)\n"
        return 0
    else
        echo_status "$ETCD_NAME" "Stopped\n"
        return 1
    fi
}

# ---------------------------------------------------------------------------

# Pass down the kill signal to exit the children
function cleanup_if_session_leader
{
    if ! ps -d -q $$ > /dev/null; then
        # We are a session leader, killing session kills all children.
        echo "trap: killing obdc session $$ $(jobs -pr)"
        kill -- -$$
    else
        # If running under job control
        local CHILD_PIDS=$(jobs -pr)
        if [ ! -z "$CHILD_PIDS" ]; then
            echo "trap: not session leader $$, jobs: $CHILD_PIDS"
            kill $CHILD_PIDS
        fi
    fi
}

trap cleanup_if_session_leader SIGINT SIGTERM

# ---------------------------------------------------------------------------

USAGE_STR="
This internal use program starts and stops the $ETCD_NAME

Usage: $0 {start|launch|stop|status|restart} [config_path] [log_filename] [debug_port] [max_memory]
"

# ---------------------------------------------------------------------------

case "$1" in

    'start')
        etcd_start "NOHUP" $2 "" $4 $5
        etcd_status
        exit $?
        ;;

    'launch')
        etcd_start "" $2 $3 $4 $5
        exit $?
        ;;

    'stop')

        etcd_stop
        exit $?
        ;;

    'restart')

        etcd_stop
        etcd_start "NOHUP" $2 "" $4 $5
        etcd_status
        exit $?
        ;;

    'status')

        etcd_status
        exit $?
        ;;

    *)
        echo "$USAGE_STR"
        ;;

esac
